Imports System.IO
Imports System.Text.RegularExpressions

Public Class FrmMain

    Private mRegEx As Regex

    Private mSemaphore As System.Threading.Semaphore

    Private mStartTime As DateTime

    Private Sub btnBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBrowse.Click
        If Me.FolderBrowserDialog1.ShowDialog = Windows.Forms.DialogResult.OK Then
            Me.txtSearchLocation.Text = Me.FolderBrowserDialog1.SelectedPath
        End If
    End Sub

    Private Sub btnSearch_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSearch.Click
        'disable search
        Me.btnSearch.Enabled = False

        'clear the results
        Me.lstResults.Items.Clear()
        Me.lbStatus.Text = String.Empty

        ' populate the argument object
        Dim args As New clsSearchArgs
        args.SearchExpression = Me.txtSearchExpression.Text.Trim
        args.SearchLocation = Me.txtSearchLocation.Text.Trim
        args.UseRegEx = Me.chkRegEx.Checked

        'record the time 
        Me.mStartTime = Now

        'start the semaphore
        Me.mSemaphore = New System.Threading.Semaphore(0, 1)

        ' run the thread
        Me.BackgroundWorker1.RunWorkerAsync(args)

        ' enable the stop button
        Me.btnStopSearch.Enabled = True

    End Sub

    Private Sub BackgroundWorker1_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundWorker1.DoWork

        'retrieve the arguments
        Dim args As clsSearchArgs = CType(e.Argument, clsSearchArgs)

        'set the regex

        If args.UseRegEx Then
            Me.mRegEx = New Regex(args.SearchExpression)
        Else
            'force lower case to make case insensitive
            args.SearchExpression = args.SearchExpression.ToLower()
        End If

        Me.DirectorySearch(args.SearchLocation, args, e)

    End Sub

    Private Sub DirectorySearch(ByVal targetdir As String, ByVal args As clsSearchArgs, ByVal e As System.ComponentModel.DoWorkEventArgs)
        'search directory for files matching args

        'check for cancellation
        If Me.BackgroundWorker1.CancellationPending Then
            'stop the search
            e.Cancel = True
            Exit Sub
        End If

        Dim results As New ArrayList

        Dim filename As String
        Dim directoryname As String

        'search this directory
        Try
            For Each filename In Directory.GetFiles(targetdir)
                Try
                    If isMatch(Path.GetFileName(filename), args) Then
                        results.Add(filename)
                    End If
                Catch exc As System.UnauthorizedAccessException
                    'continue

                End Try

                If results.Count > 10 Then
                    Me.ReportTheProgress(results)
                    results = New ArrayList
                End If
            Next
        Catch exc As System.ArgumentException
            'invalid directory?
            Exit Sub
        End Try

        'report progress
        Me.ReportTheProgress(results)
        results = New ArrayList

        'search the subdirectories
        For Each directoryname In Directory.GetDirectories(targetdir)

            Try
                DirectorySearch(directoryname, args, e)
            Catch exc As System.UnauthorizedAccessException
                'skip this one
            End Try
        Next

    End Sub

    Private Function isMatch(ByVal filename As String, ByVal args As ClsSearchArgs) As Boolean

        Dim sFileName As String = filename.ToLower
        Dim chWild As Char = Chr(42) ' *

        If args.UseRegEx Then
            Dim m As Match = mRegEx.Match(filename)
            Return m.Success
        Else
            If args.SearchExpression.StartsWith(chWild) Then
                If args.SearchExpression.EndsWith(chWild) Then
                    Dim s As String = args.SearchExpression.TrimEnd(chWild).TrimStart(chWild)
                    Return (sFileName.IndexOf(s) > -1)
                Else
                    Return sFileName.EndsWith(args.SearchExpression.Substring(1))
                End If
            ElseIf args.SearchExpression.EndsWith(chWild) Then
                Return sFileName.StartsWith(args.SearchExpression.TrimEnd(chWild))
            Else
                Return (sFileName = args.SearchExpression)
            End If
        End If


    End Function

    Private Sub ReportTheProgress(ByVal results As ArrayList)

        If results.Count > 0 Then

            Me.BackgroundWorker1.ReportProgress(0, results)
            Me.mSemaphore.WaitOne(1000, False)
            'have a break
            'System.Threading.Thread.Sleep(1)
            'Me.UpdateList(results)
            'System.Threading.Thread.Sleep(1)
        End If

    End Sub


    Private Sub BackgroundWorker1_RunWorkerCompleted(ByVal sender As System.Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundWorker1.RunWorkerCompleted
        Me.btnStopSearch.Enabled = False
        If e.Cancelled Then
            Me.lbStatus.Text = Me.lbStatus.Text + " search cancelled"
        ElseIf Not IsNothing(e.Error) Then
            Me.lbStatus.Text = Me.lbStatus.Text + " error in search"
        Else
            Me.lbStatus.Text = Me.lbStatus.Text + " search done"
        End If

        Me.lbTime.Text = "Time taken: " + Format((Now - Me.mStartTime).TotalSeconds, "n") + " seconds"

        Me.btnSearch.Enabled = True
    End Sub

    Private Sub BackgroundWorker1_ProgressChanged(ByVal sender As System.Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundWorker1.ProgressChanged
        'update the listbox

        Try
            Dim results As ArrayList = CType(e.UserState, ArrayList)
            Dim o As Object

            lstResults.BeginUpdate()

            For Each o In results
                Dim s As String = CType(o, String)
                Me.lstResults.Items.Add(s)
            Next

            lstResults.EndUpdate()

            Me.lbStatus.Text = lstResults.Items.Count.ToString + " matching files"
        Finally ' always release the semaphore
            Me.mSemaphore.Release()
        End Try

    End Sub

    Private Sub btnStopSearch_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStopSearch.Click
        Me.BackgroundWorker1.CancelAsync()
    End Sub

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
    End Sub
End Class


